//
//  SenzorstableViewController.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 05.10.17.
//  Copyright © 2017 Lukáš Šanda. All rights reserved.
//

import UIKit
import SnapKit


// MARK: - Controller pro tableView
/// Třída obalující práci se SenzortableView
class SenzorsTableController: BaseViewController, UITableViewDelegate, UITableViewDataSource {
    
    // MARK: - Proměnné
    
    var senzors = [Senzor]()
    var tableView = UITableView()
    
    // MARK: - Funkce
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
        
        view.showBlurLoader()
        
        // Naplnění tableView daty získaných pomocí REST API
        SenzorAPI.shared.requestSenzors { (senzors, error) in
            
            // Pokud nenastala chyba při získávání dat, data se nastaví do paměti aplikace a proběhne naplnění tableView
            if error == nil {
                if let senzors = senzors {
                    VodomerData.shared.senzors = senzors
                    
                    VodomerData.shared.totalConsumption(completionHandler: { (error) in
                        if error == nil {
                            self.senzors = senzors
                            
                            self.tableView.reloadData()
                            self.view.removeBlurLoader()
                        } else {
                            self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                        }
                    })
                }
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.title = "COLLECTION_TITLE".localized
        initializeTableView()
    }
    
    // MARK: - Delegované metody komponenty UITableView
    
    // MARK: Funkce, která vrací počet sekcí pro tableView
    func numberOfSections(in tableView: UITableView) -> Int {
        tableView.isHidden = senzors.isEmpty ? true : false
        
        return senzors.isEmpty ? 0 : 1
    }
    
    // MARK: Funkce, která vrací velikost řádku
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableViewAutomaticDimension
    }
    
    // MARK: Funkce, která vrací záznamů v tableView
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return senzors.count
    }
    
    // MARK: Funkce, která nastavuje hodnoty danému záznamu
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: SenzorCell =  tableView.dequeueReusableCell(for: indexPath)
        let senzor = senzors[indexPath.row]
        
        cell.configure(for: senzor)
        
        return cell
    }
    
    // MARK: Funkce, která při výběru záznamu zařídí přechod mezi View.
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        view.showBlurLoader()
        tableView.isUserInteractionEnabled = false
        
        let storyboard: UIStoryboard = .detail
        let viewController = storyboard.instantiateViewController(withIdentifier: "SenzorViewController") as! SenzorViewController
        viewController.senzor = self.senzors[indexPath.row]
        
        self.tableView.isUserInteractionEnabled = true
        self.show(viewController, sender: nil)
    }
    
    // MARK: - Privátní funkce
    
    /// Funkce pro zachycení události kliknutí na tlačítko "Odhlásit" v navigační liště
    ///
    /// - Parameter sender: Objekt, který událost odeslal
    @IBAction private func signout(_ sender: UIBarButtonItem) {
        let storyboard: UIStoryboard = .main
        let viewController = storyboard.instantiateViewController(withIdentifier: "LoginViewController") as! LoginViewController
        
        UserDefaults.standard.userLoginToken.removeAll()
        show(viewController, sender: nil)
    }
    
    /// Funkce pro inicializaci komponent grafického rozhraní
    private func initializeTableView() {
        
        tableView = UITableView(frame: .zero, style: .plain)
        tableView.tableFooterView = UIView(frame: .zero)
        tableView.delegate = self
        tableView.dataSource = self
        
        tableView.separatorColor = .clear
        tableView.estimatedRowHeight = 100
        tableView.backgroundColor = .primaryColor
        
        tableView.registerReusableNibCell(for: SenzorCell.self)
        self.navigationItem.hidesBackButton = true
        
        view.addSubview(tableView)
        tableView.snp.makeConstraints {
            $0.edges.equalToSuperview()
        }
        
        let refreshControl = UIRefreshControl()
        refreshControl.tintColor = .black
        tableView.refreshControl = refreshControl
        refreshControl.addTarget(self, action: #selector(refresh), for: .valueChanged)
        
    }
    
    
    /// Funkce pro zachycení události kliknutí na tlačítko "Obnovit"
    ///
    /// - Parameter sender: Objekt, který událost zaslal
    @IBAction private func buttonRefreshTouched(_ sender: UIBarButtonItem) {
        self.refresh()
    }
    
    /// Pomocná funkce, která vykonává obnovu tabulky
    ///
    /// - Parameter sender: Objekt, který událost zaslal
    @objc private func refresh(sender: UIRefreshControl? = nil) {
        
        SenzorAPI.shared.requestSenzors { (senzors, error) in
            // Pokud nenastala chyba při získávání dat, data se nastaví do paměti aplikace a proběhne naplnění tableView
            if error == nil {
                if let senzors = senzors  {
                    VodomerData.shared.senzors = senzors
                    self.senzors = senzors
                    
                    VodomerData.shared.totalConsumption(completionHandler: { (error) in
                        if error == nil {
                            self.senzors = VodomerData.shared.senzors
                            self.tableView.reloadData()
                            
                            if let sender = sender {
                                sender.endRefreshing()
                            }
                        } else {
                            self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                        }
                    })
                }
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
            }
        }
    }
}
