//
//  LoginViewController.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 16.01.18.
//  Copyright © 2018 Lukáš Šanda. All rights reserved.
//

import UIKit


// MARK: - Třída spravující View pro přihlášení do aplikace
/// Třída spravující View pro přihlášení do aplikace
class LoginViewController: UIViewController, UITextFieldDelegate {
    
    // MARK: - Proměnné
    
    @IBOutlet var textFieldUsername: UITextField!
    @IBOutlet var textFieldPassword: UITextField!
    @IBOutlet weak var btnLogin: UIButton!
    
    // MARK: - Funkce
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        btnLogin.backgroundColor = .primaryColor
        btnLogin.tintColor = .white
        self.navigationController?.isNavigationBarHidden = true
        
        textFieldUsername.delegate = self
        textFieldPassword.delegate = self
        
        textFieldUsername.placeholder = "LOGIN_USERNAME_PLACEHOLDER".localized
        textFieldPassword.placeholder = "LOGIN_PASSWORD_PLACEHOLDER".localized
        btnLogin.setTitle("LOGIN_BUTTON_LOGIN".localized, for: .normal)
    }
    
    
    // MARK: -  Delegovaná funkce komponenty UITextField, pro zachycení události kliknutí na tlačítko "Enter" na klávesnici
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true
    }
    
    // MARK: - Privátní Funkce
    
    /// Funkce pro zachycení události kliknutí na tlačítko "Přihlásit"
    ///
    /// - Parameter sender: Objekt, který událost zaslal
    @IBAction private func login(_ sender: UIButton) {
        view.showBlurBackground()
        
        let username = textFieldUsername.text ?? ""
        let password = textFieldPassword.text ?? ""
        
        AuthenticationAPI.shared.login(username: username, password: password) { (token, error) in
            self.view.removeBlurBackground()
            
            if error == nil {
                // Uložení tokenu do lokální paměti
                if let token = token {
                    UserDefaults.standard.userLoginToken = token
                }
                
                // Dekódování role uživatele
                JWTManager.shared.decode()
                
                // Přechod na kolekci vodoměrů
                let storyboard: UIStoryboard = .collection
                let collectionView = storyboard.instantiateViewController(withIdentifier: "collectionNavigationController")
                
                self.present(collectionView, animated: true, completion: nil)
            } else {
                //Pokud není k dispozici internetové připojení či byly zadány špatně přihlašovací údaje
                if ReachabilityManager.shared.isConnectedToNetwork() {
                    self.showWrongCredentialsAlert()
                } else {
                    self.showNoConnectionAlert()
                }
            }
        }
    }
    
    /// Funkce zajišťující zobrazení alertu při absenci připojení k internetu
    private func showNoConnectionAlert() {
        let noConnectionAlertView = UIAlertController(title: "LOGIN_ALERT_NO_CONNECTION_TITLE".localized, message: "LOGIN_ALERT_NO_CONNECTION_MESSAGE".localized, preferredStyle: .alert)
        noConnectionAlertView.addAction(UIAlertAction(title: "LOGIN_ALERT_NO_CONNECTION_BUTTON_TITLE".localized, style: .cancel, handler: {
            (action) in
            self.view.removeBlurBackground()
        }))
        
        present(noConnectionAlertView, animated: true, completion: nil)
    }
    
    /// Funkce zajišťující zobrazení alertu při špatně zadaných údajích
    private func showWrongCredentialsAlert() {
        let alertWrongCredentials = UIAlertController(title: "LOGIN_ALERT_WRONG_CREDENTIALS_TITLE".localized, message: "LOGIN_ALERT_WRONG_CREDENTIALS_MESSAGE".localized, preferredStyle: .actionSheet)
        
        alertWrongCredentials.addAction(UIAlertAction(title: "LOGIN_ALERT_WRONG_CREDENTIALS_BUTTON_UNDERSTAND_TITLE".localized, style: .cancel, handler: nil))
        alertWrongCredentials.addAction(UIAlertAction(title: "LOGIN_ALERT_WRONG_CREDENTIALS_BUTTON_PASSWORD_TITLE".localized, style: .default, handler: { _ in
            guard let url = URL(string: "http://petrkoula-001-site1.ctempurl.com/user?ReturnUrl=%2F") else {
                return
            }
            
            if #available(iOS 10.0, *) {
                UIApplication.shared.open(url, options: [:], completionHandler: nil)
            } else {
                UIApplication.shared.openURL(url)
            }
        }))
        self.present(alertWrongCredentials, animated: true, completion: nil)
    }
}
