//
//  SideMenuTableViewController.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 19.12.17.
//  Copyright © 2017 Lukáš Šanda. All rights reserved.
//

import UIKit

class SideMenuTableViewController: BaseTableViewController {
    
    // MARK: - Proměnné

    private var role = UserDefaults.standard.userRole
    private var senzor: Senzor! = nil
    
    // MARK: - Funkce
    
    override func viewWillAppear(_ animated: Bool) {
        self.senzor = VodomerData.shared.tempSenzor
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.registerReusableNibCell(for: MenuCell.self)
        tableView.estimatedSectionHeaderHeight = 44
        tableView.estimatedRowHeight = 60
        tableView.tableFooterView = UIView(frame: .zero)
        
        NotificationCenter.default.addObserver(self, selector: #selector(captureSelectedSenzor(notification:)), name: .menuWillAppear, object: nil)
        
        self.view.backgroundColor = .primaryColor
        self.navigationController?.navigationBar.barTintColor = .primaryColor
        self.navigationController?.view.backgroundColor = .primaryColor
        
        self.title = "MENU_TITLE".localized
    }
    
    // MARK: - Delegované funkce komponenty UITableView
    
    override func numberOfSections(in tableView: UITableView) -> Int {
        return role == "user" ? 2 : 3
    }
    
    override func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return UITableViewAutomaticDimension
    }
    
    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        switch section {
        case 0:
            return "MENU_BASIC_OPERATIONS".localized
        case 1:
            return role == "user" ? "MENU_USER_OPERATIONS".localized : "MENU_SERVICE_OPERATIONS".localized
        case 2:
            return "MENU_USER_OPERATIONS".localized
        default:
            return nil
        }
    }
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return section == 0 ? 3 : 1
    }
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableViewAutomaticDimension
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: MenuCell = tableView.dequeueReusableCell(for: indexPath)
        
        switch indexPath.section {
        case 0:
            switch indexPath.row {
            case 0:
                cell.iconImage.image = #imageLiteral(resourceName: "Collection")
                cell.labelTitle.text = "MENU_COLLECTION".localized
            case 1:
                cell.iconImage.image = #imageLiteral(resourceName: "Edit")
                cell.labelTitle.text = "MENU_EDIT".localized
            case 2:
                cell.iconImage.image = #imageLiteral(resourceName: "Spanner")
                cell.labelTitle.text = "MENU_VALVE".localized
            default:
                break
            }
        case 1:
                cell.iconImage.image = role == "user" ? #imageLiteral(resourceName: "Logout") : #imageLiteral(resourceName: "Service")
                cell.labelTitle.text = role == "user" ? "MENU_LOGOUT".localized : "MENU_SERVICE".localized
        case 2:
            cell.iconImage.image = #imageLiteral(resourceName: "Logout")
            cell.labelTitle.text = "MENU_LOGOUT".localized
        default:
            break
        }
        
        return cell
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let storyboard: UIStoryboard
        
        switch indexPath.section {
        // Základní položky
        case 0:
            switch indexPath.row {
            case 0:
                storyboard = .collection
                let viewController = storyboard.instantiateViewController(withIdentifier: "SenzorsTableController") as! SenzorsTableController
                show(viewController, sender: nil)
            case 1:
                storyboard = .edit
                let viewController = storyboard.instantiateViewController(withIdentifier: "SenzorEditTableViewController") as! SenzorEditTableViewController
                viewController.senzor = self.senzor
                show(viewController, sender: nil)
            case 2:
                storyboard = .valve
                let viewController = storyboard.instantiateViewController(withIdentifier: "ValveStateViewController") as! ValveStateViewController
                viewController.senzor = self.senzor
                show(viewController, sender: nil)
            default:
                return
            }
        // Servis
        case 1:
            if role == "user" {
                // Obnovení všech dočasně uložených hodnot
                Utils.shared.resetApplication()
                
                storyboard = .main
                let viewController = storyboard.instantiateViewController(withIdentifier: "LoginViewController") as! LoginViewController
                show(viewController, sender: nil)
            } else {
                storyboard = .service
                let viewController = storyboard.instantiateViewController(withIdentifier: "ServiceTableViewController") as! ServiceTableViewController
                viewController.senzor = self.senzor
                show(viewController, sender: nil)
            }
        // Uživatelské položky
        case 2:
            // Obnovení všech dočasně uložených hodnot
            Utils.shared.resetApplication()
            
            storyboard = .main
            let viewController = storyboard.instantiateViewController(withIdentifier: "LoginViewController") as! LoginViewController
            show(viewController, sender: nil)
        default:
            return
        }
    }
    
    // MARK: - Privátní funkce
    
    /// Funkce pro zachycení notifikace, která obsahuje senzor, který byl vybrán z tabulky senzorů
    ///
    /// - Parameter sender: Zaslaná notifikace
    @objc private func captureSelectedSenzor(notification: Notification) {
        if let capturedSenzor = notification.userInfo?["senzor"] as? Senzor {
            self.senzor = capturedSenzor
        }
    }
}
