//
//  ConsumptionPieChart.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 04.02.18.
//  Copyright © 2018 Lukáš Šanda. All rights reserved.
//

import Foundation
import Charts


// MARK: - Pomocná funkce pro vytvoření koláčového grafu
class ConsumptionPieChart {
    
    // MARK: - Proměnné
    
    let senzor: Senzor
    var pieChart: PieChartView
    
    init(_ pieChart: PieChartView, for senzor: Senzor) {
        self.senzor = senzor
        self.pieChart = pieChart
    }
    
    
    // MARK: - Funkce
    
    /// Funkce pro vytvoření koláčového grafu
    ///
    /// - Parameter type: Období, pro které má být graf vytvořen
    /// - Returns: Vytvořený graf
    func createPieChart(type: TimeUnit) -> PieChartView? {
        var pieChartEntry = [PieChartDataEntry]()
        let consumption = Utils.shared.calcConsumption(type, for: senzor)
        let val = PieChartDataEntry(value: consumption, label: "") // Spotřeba
        pieChartEntry.append(val)
        
        var limitVal = 0.0
        var limit: PieChartDataEntry
        
        if type == .day {
            limitVal = senzor.limitDay! - consumption
            limit = PieChartDataEntry(value: limitVal < 0 ? 0 : limitVal, label: "") // Limit
        } else {
            limitVal = senzor.limitMonth! - consumption
            limit = PieChartDataEntry(value: limitVal < 0 ? 0 : limitVal, label: "")
        }
        pieChartEntry.append(limit)
        
        let dataSet = PieChartDataSet(values: pieChartEntry, label: "DETAIL_PIECHART_DESCRIPTION".localized)
        // Skrytí hodnot části grafu
        dataSet.drawValuesEnabled = false
        dataSet.colors = [.darkGray, .primaryColor]
        
        let data = PieChartData()
        data.addDataSet(dataSet)
        
        let percent = (100 / (type == .day ? senzor.limitDay : senzor.limitMonth)!) * consumption
        let attribute = [ NSAttributedStringKey.font: UIFont(name: "Arial", size: 15.0)! ]
        pieChart.centerAttributedText = NSAttributedString(string: "\(String(format: "%.1f", percent)) %", attributes: attribute)
        pieChart.animate(yAxisDuration: 2)
        pieChart.data = data
        
        pieChartAppereance()
        
        return pieChart
    }
    
    /// Pomocná funkce, která spravuje vizuální stránku grafu
    private func pieChartAppereance() {
        pieChart.legend.enabled = false
        pieChart.chartDescription?.enabled = false
        pieChart.usePercentValuesEnabled = true
    }
}
