  //
  //  SenzorViewController.swift
  //  Hydrometra
  //
  //  Created by Lukáš Šanda on 01.10.17.
  //  Copyright © 2017 Lukáš Šanda. All rights reserved.
  //
  
  import UIKit
  import Charts
  import SideMenu
  
  
  // MARK: - Controller pro SenzorView
  /// Třída spravující vše spadající pod detailní pohled na vodoměr.
  class SenzorViewController: BaseViewController {
    
    // MARK: - Proměnné
    
    @IBOutlet weak var labelDayConsumptionTitle: ChartTitle!
    @IBOutlet weak var labelMonthConsumptionTitle: ChartTitle!
    
    @IBOutlet weak var barChartView: BarChartView!
    @IBOutlet weak var dayPieChartView: PieChartView!
    @IBOutlet weak var monthPieChartView: PieChartView!
    
    @IBOutlet weak var labelDayCurrentConsumption: UILabel!
    @IBOutlet weak var labelDayLimit: UILabel!
    @IBOutlet weak var labelDayConsumption: UILabel!
    
    @IBOutlet weak var labelMonthCurrentConsumption: UILabel!
    @IBOutlet weak var labelMonthLimit: UILabel!
    @IBOutlet weak var labelMonthConsumption: UILabel!
    
    /// Label u sloupcové grafu
    @IBOutlet weak var labelBarChart: UILabel!
    
    /// Zvolený senzor z CollectionView
    var senzor: Senzor! = nil
    var selectedType: TimeUnit = .day
    
    // Date Picker
    @IBOutlet weak var segmentButtons: UISegmentedControl!
    
    @IBOutlet weak var btnOffsetLeft: UIButton!
    @IBOutlet weak var btnOffsetRight: UIButton!
    @IBOutlet weak var btnPeriodUp: UIButton!
    
    @IBOutlet weak var tfDatePicker: UITextField!
    
    let datePicker = UIDatePicker()
    let dateFormatter = DateFormatter()
    
    var barChartController: ConsumptionBarChart?
    var dayPieChartController: ConsumptionPieChart?
    var monthPieChartController: ConsumptionPieChart?
    
    var storedDate = Date()
    
    // MARK: - Funkce
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
        
        view.showBlurLoader()
        
        self.view.backgroundColor = .secondaryColor
        
        // Při inicializaci view se provede request na spotřebu
        VodomerData.shared.defaultInitConsumption(id: senzor.idHw!) { (senzor, error) in
            if error == nil {
                if let senzor = senzor {
                    VodomerData.shared.tempSenzor = senzor
                    self.senzor = senzor
                    self.barChartController = ConsumptionBarChart(self.barChartView, for: senzor)
                    self.dayPieChartController = ConsumptionPieChart(self.dayPieChartView, for: senzor)
                    self.monthPieChartController = ConsumptionPieChart(self.monthPieChartView, for: senzor)
                    
                    // Naplnění hodnot do příslušných Labels
                    self.fillLabels()
                    
                    // Inicializace grafů
                    self.updateBarChart()
                    self.updatePieChart(type: .day)
                    self.updatePieChart(type: .month)
                    
                    self.resetDate()
                    self.view.removeBlurLoader()
                }
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: true), animated: true, completion: nil)
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        dateFormatter.locale = Locale(identifier: NSLocale.current.languageCode!)
        
        // Delegate
        barChartView.delegate = self
        
        viewAppereance()
        
        let storyboard: UIStoryboard = .menu
        let viewController = storyboard.instantiateViewController(withIdentifier: "SideMenuNavigationController") as! UISideMenuNavigationController
        
        SideMenuManager.default.menuRightNavigationController = viewController
        SideMenuManager.default.menuAddPanGestureToPresent(toView: self.navigationController!.navigationBar)
        SideMenuManager.default.menuAddScreenEdgePanGesturesToPresent(toView: self.navigationController!.view)
        SideMenuManager.default.menuPresentMode = .menuSlideIn
        SideMenuManager.default.menuFadeStatusBar = false
        
        localizedStrings()
        
        let backButton = UIBarButtonItem(title: "BUTTON_BACK".localized, style: .done, target: self, action: #selector(backButtonAction))
        self.navigationItem.leftBarButtonItem = backButton
    }
    
    @objc private func backButtonAction() {
        let storyboard: UIStoryboard = .collection
        let viewController = storyboard.instantiateViewController(withIdentifier: "SenzorsTableController") as! SenzorsTableController
        
        self.show(viewController, sender: nil)
    }
    
    // MARK: Funkce obslužující spojnicový graf
    /// Funkce obslužující spojnicový graf
    func updateBarChart() {
        barChartView = barChartController?.createBarChart(type: selectedType) //barChartController?.emptyChart(barChart: barChartView)
        barChartView.animate(yAxisDuration: 1.5)
        
        barChartView.notifyDataSetChanged()
    }
    
    // MARK: - Privátní funkce
    
    /// Funkce zajišťující naplnění lokalizovaných textů v GUI
    private func localizedStrings() {
        labelDayConsumptionTitle.text = "DETAIL_DAY_CONSUMPTION".localized
        labelMonthConsumptionTitle.text = "DETAIL_MONTH_CONSUMPTION".localized
        
        segmentButtons.setTitle("SEGMENT_DAY".localized, forSegmentAt: 0)
        segmentButtons.setTitle("SEGMENT_MONTH".localized, forSegmentAt: 1)
        segmentButtons.setTitle("SEGMENT_YEAR".localized, forSegmentAt: 2)
        
        labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_DAY".localized
    }
    
    // MARK: Funkce spravující vizuální stránku a její doplňující prvky
    /// Funkce spravující vizuální stránku a její doplňující prvky
    private func viewAppereance() {
        // Nastavení titulního textu
        self.title = senzor.nameHw
        
        segmentButtons.tintColor = .primaryColor
        btnPeriodUp.tintColor = .primaryColor
        btnOffsetLeft.tintColor = .primaryColor
        btnOffsetRight.tintColor = .primaryColor
    }
    
    // MARK: Funkce pro naplnění hodnot do UILabels
    /// Funkce pro naplnění hodnot do UILabels
    private func fillLabels() {
        
        // Labels pro graf DayPieChartView
        labelDayCurrentConsumption.text = { () -> String in
            var hoursCons = Utils.shared.calcConsumption(.day, for: senzor)
            hoursCons *= senzor.pricePerM3!
            return "\(String(format: "%.0f", hoursCons)) Kč"
        }()
        if let limitDay = senzor.limitDay {
            labelDayLimit.text = "DETAIL_LABEL_LIMIT".localized + " \(String(format: "%.2f", limitDay)) m³"
        }
        labelDayConsumption.text = "DETAIL_LABEL_CONSUMPTION".localized + " \(String(format: "%.2f", Utils.shared.calcConsumption(.day, for: senzor))) m³"
        
        // Labels for MonthPieChartView
        labelMonthCurrentConsumption.text =  { () -> String in
            var monthCons = Utils.shared.calcConsumption(.month, for: senzor)
            monthCons *= senzor.pricePerM3!
            return "\(String(format: "%.0f", monthCons)) Kč"
        }()
        if let limitMonth = senzor.limitMonth {
            labelMonthLimit.text = "DETAIL_LABEL_LIMIT".localized + " \(String(format: "%.2f", limitMonth)) m³"
        }
        labelMonthConsumption.text = "DETAIL_LABEL_CONSUMPTION".localized + " \(String(format: "%.2f", Utils.shared.calcConsumption(.month, for: senzor))) m³"
    }
    
    // MARK: Funkce obslužující koláčový graf
    /// Funkce obslužující koláčový graf
    ///
    /// - Parameter type: Požadované období, ze kterého bude graf sestaven
    private func updatePieChart(type: TimeUnit) {
        switch type {
        case .day:
            dayPieChartView = dayPieChartController?.createPieChart(type: type)
            dayPieChartView.notifyDataSetChanged()
        default:
            monthPieChartView = monthPieChartController?.createPieChart(type: type)
            monthPieChartView.notifyDataSetChanged()
        }
    }
    
    /// Funkce pro zachycení události kliknutí na tlačítko změny offsetu do záporné hodnoty
    ///
    /// - Parameter sender: Objekt, který událost vyslal
    @IBAction private func btnOffsetMinus(_ sender: UIButton) {
        self.dateFormatter.dateFormat = "YYYY-MM-dd'T'HH:mm:ss'Z'"
        let date: String = dateFormatter.string(from: storedDate)
        
        //  Požadavek na data pro předchozí den k uloženému datu
        ConsumptionAPI.shared.requestConsumption(id: senzor.idHw!, offset: -1, timeUnit: selectedType, date: date) { (senzor, error) in
            if error == nil {
                switch self.selectedType {
                case .day:
                    self.dateFormatter.dateFormat = "dd. MM. YYYY"
                    self.labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_DAY".localized
                    
                    let date = Utils.shared.getDateBasedOnOffset(timeUnit: .day, -1, date: self.storedDate)!
                    self.storedDate = date
                    self.tfDatePicker.text = self.dateFormatter.string(from: date)
                    
                    self.senzor.consumptionDay = senzor?.consumptionDay
                    self.updateBarChart()
                    
                case .month:
                    self.dateFormatter.dateFormat = "MMMM, YYYY"
                    self.labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_MONTH".localized
                    
                    let date = Utils.shared.getDateBasedOnOffset(timeUnit: .month, -1, date: self.storedDate)!
                    self.storedDate = date
                    self.tfDatePicker.text = self.dateFormatter.string(from: date)
                    
                    self.senzor.consumptionMonth = senzor?.consumptionMonth
                    self.updateBarChart()
                    
                case .year:
                    self.dateFormatter.dateFormat = "YYYY"
                    self.labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_YEAR".localized
                    
                    let date = Utils.shared.getDateBasedOnOffset(timeUnit: .year, -1, date: self.storedDate)!
                    self.storedDate = date
                    self.tfDatePicker.text = self.dateFormatter.string(from: date)
                    
                    self.senzor.consumptionYear = senzor?.consumptionYear
                    self.updateBarChart()
                }
                
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
            }
        }
    }
    
    /// Funkce pro zachycení události kliknutí na tlačítko změny offsetu do kladné hodnoty
    ///
    /// - Parameter sender: Objekt, který událost vyslal
    @IBAction private func btnPeriodUp(_ sender: UIButton) {
        let calendar = Calendar.current
        
        let yearCal = calendar.component(.year, from: self.storedDate)
        let monthCal = calendar.component(.month, from: self.storedDate)
        let dayCal = calendar.component(.day, from: self.storedDate)
        
        switch selectedType {
        case .day:
            selectedType = .month
            segmentButtons.selectedSegmentIndex = 1
            dateFormatter.dateFormat = "MMMM, YYYY"
            let month: String = dateFormatter.string(from: storedDate)
            tfDatePicker.text = month
            
            let dateStr = "\(yearCal)-\(monthCal)-\(dayCal)"
            self.dateFormatter.dateFormat = "yyyy-MM-dd"
            let date = dateFormatter.date(from: dateStr)
            self.dateFormatter.dateFormat = "YYYY-MM-dd'T'HH:mm:ss'Z'"
            let dateString = dateFormatter.string(from: date!)
            storedDate = dateFormatter.date(from: dateString)!
            
            labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_YEAR".localized
            ConsumptionAPI.shared.requestConsumption(id: senzor.idHw!, offset: 0, timeUnit: .month, date: dateString, completionHandler: { (senzor, error) in
                if error == nil {
                    for senzor in VodomerData.shared.senzors {
                        if senzor.idHw == self.senzor.idHw {
                            self.senzor.consumptionMonth = senzor.consumptionMonth
                            self.updateBarChart()
                            return
                        }
                    }
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                }
            })
            
        case .month:
            selectedType = .year
            segmentButtons.selectedSegmentIndex = 2
            dateFormatter.dateFormat = "YYYY"
            let year: String = dateFormatter.string(from: storedDate)
            tfDatePicker.text = year
            
            let dateStr = "\(yearCal)-\(monthCal)-\(dayCal)"
            self.dateFormatter.dateFormat = "yyyy-MM-dd"
            let date = dateFormatter.date(from: dateStr)
            self.dateFormatter.dateFormat = "YYYY-MM-dd'T'HH:mm:ss'Z'"
            let dateString = dateFormatter.string(from: date!)
            storedDate = dateFormatter.date(from: dateString)!
            
            labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_YEAR".localized
            ConsumptionAPI.shared.requestConsumption(id: senzor.idHw!, offset: 0, timeUnit: .year, date: dateString, completionHandler: { (senzor, error) in
                if error == nil {
                    for senzor in VodomerData.shared.senzors {
                        if senzor.idHw == self.senzor.idHw {
                            self.senzor.consumptionYear = senzor.consumptionYear
                            self.updateBarChart()
                            return
                        }
                    }
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                }
            })
            
        default:
            return
        }
    }
    
    /// Funkce pro zachycení události kliknutí na tlačítko změny offsetu o řád výš
    ///
    /// - Parameter sender: Objekt, který událost vyslal
    @IBAction func btnOffsetPlus(_ sender: UIButton) {
        self.dateFormatter.dateFormat = "YYYY-MM-dd'T'HH:mm:ss'Z'"
        let date: String = dateFormatter.string(from: storedDate)
        
        ConsumptionAPI.shared.requestConsumption(id: senzor.idHw!, offset: 1, timeUnit: selectedType, date: date) { (senzor, error) in
            if error == nil {
                switch self.selectedType {
                case .day:
                    self.dateFormatter.dateFormat = "dd. MM. YYYY"
                    self.labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_DAY".localized
                    
                    let date = Utils.shared.getDateBasedOnOffset(timeUnit: .day, 1, date: self.storedDate)!
                    self.storedDate = date
                    self.tfDatePicker.text = self.dateFormatter.string(from: date)
                    
                    self.senzor.consumptionDay = senzor?.consumptionDay
                    self.updateBarChart()
                    
                case .month:
                    self.dateFormatter.dateFormat = "MMMM, YYYY"
                    self.labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_MONTH".localized
                    
                    let date = Utils.shared.getDateBasedOnOffset(timeUnit: .month, 1, date: self.storedDate)!
                    self.storedDate = date
                    self.tfDatePicker.text = self.dateFormatter.string(from: date)
                    
                    self.senzor.consumptionMonth = senzor?.consumptionMonth
                    self.updateBarChart()
                    
                case .year:
                    self.dateFormatter.dateFormat = "YYYY"
                    self.labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_YEAR".localized
                    
                    let date = Utils.shared.getDateBasedOnOffset(timeUnit: .year, 1, date: self.storedDate)!
                    self.storedDate = date
                    self.tfDatePicker.text = self.dateFormatter.string(from: date)
                    
                    self.senzor.consumptionYear = senzor?.consumptionYear
                    self.updateBarChart()
                }
                
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
            }
        }
    }
    
    // MARK: - Pomocná funkce, která nastaví do lokální proměnné aktuální datum
    /// Pomocná funkce, která nastaví do lokální proměnné aktuální datum
    private func resetDate() {
        dateFormatter.dateFormat = "dd. MM. YYYY"
        storedDate = Date()
        tfDatePicker.text = dateFormatter.string(from: storedDate)
    }
    
    // MARK: Funkce spravující akce pro tlačítko "Menu"
    /// Funkce pro zachycení události kliknutí na tlačítko "Menu"
    ///
    /// - Parameter sender: Objekt, který událost vyslal
    @IBAction func btnMenu(_ sender: UIBarButtonItem) {
        if let menu = SideMenuManager.defaultManager.menuRightNavigationController {
            present(menu, animated: true) {
                NotificationCenter.default.post(name: .menuWillAppear, object: nil, userInfo: ["senzor": self.senzor])
            }
        }
    }
    
    // MARK: Funkce provádějící naplnění hodnot do grafu po kliknutí na segment, reprezentující dané období
    /// Funkce pro zachycení události kliknutí na tlačítko v komponentě UISegmentControl
    ///
    /// - Parameter sender: Objekt, který událost vyslal
    @IBAction func periodStateChange(_ sender: UISegmentedControl) {
        resetDate()
        
        switch sender.selectedSegmentIndex {
        case 0:
            selectedType = .day
            dateFormatter.dateFormat = "dd. MM. YYYY"
            let date: String = dateFormatter.string(from: storedDate)
            tfDatePicker.text = date
            
            labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_DAY".localized
            ConsumptionAPI.shared.requestConsumption(id: senzor.idHw!, offset: 0, timeUnit: .day, completionHandler: { (senzor, error) in
                if error == nil {
                    self.senzor.consumptionDay = VodomerData.shared.senzors.first(where: { $0.idHw == senzor?.idHw })?.consumptionDay
                    
                    self.updateBarChart()
                    return
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                }
            })
            
        case 1:
            selectedType = .month
            dateFormatter.dateFormat = "MMMM, YYYY"
            let month: String = dateFormatter.string(from: storedDate)
            tfDatePicker.text = month
            
            labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_MONTH".localized
            ConsumptionAPI.shared.requestConsumption(id: senzor.idHw!, offset: 0, timeUnit: .month, completionHandler: { (senzor, error) in
                if error == nil {
                    self.senzor.consumptionMonth = VodomerData.shared.senzors.first(where: { $0.idHw == senzor?.idHw })?.consumptionMonth
                    
                    self.updateBarChart()
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                }
            })
            
        case 2:
            selectedType = .year
            dateFormatter.dateFormat = "YYYY"
            let year: String = dateFormatter.string(from: storedDate)
            tfDatePicker.text = year
            labelBarChart.text = "DETAIL_BARCHART_DESCRIPTION_YEAR".localized
            ConsumptionAPI.shared.requestConsumption(id: senzor.idHw!, offset: 0, timeUnit: .year, completionHandler: { (senzor, error) in
                if error == nil {
                    self.senzor.consumptionDay = VodomerData.shared.senzors.first(where: { $0.idHw == senzor?.idHw })?.consumptionYear
                    
                    self.updateBarChart()
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                }
            })
            
        default:
            return
        }
    }
    
  }
