//
//  SenzorEditTableViewController.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 21.11.17.
//  Copyright © 2017 Lukáš Šanda. All rights reserved.
//

import UIKit
import Alamofire


// MARK: - Třída spravující View pro editovatelných hodnot Vodoměru
/// Třída zajišťuje veškeré dění pro View pro editaci hodnot vodoměru.
class SenzorEditTableViewController: BaseTableViewController, UITextFieldDelegate {
    
    // MARK: - Proměnné
    
    var senzor: Senzor! = nil
    
    @IBOutlet weak var labelPricePerM3: tableHeaderTitle!
    @IBOutlet weak var labelStartNight: tableHeaderTitle!
    @IBOutlet weak var labelStartNightMinutes: tableHeaderTitle!
    @IBOutlet weak var labelEndNight: tableHeaderTitle!
    @IBOutlet weak var labelEndNightMinutes: tableHeaderTitle!
    @IBOutlet weak var labelCountStop: tableHeaderTitle!
    @IBOutlet weak var labelCountStopNight: tableHeaderTitle!
    @IBOutlet weak var labelDayLimit: tableHeaderTitle!
    @IBOutlet weak var labelMonthLimit: tableHeaderTitle!
    
    
    @IBOutlet weak var tfSenzorName: UITextField!
    @IBOutlet weak var tfPricePerM3: UITextField!
    @IBOutlet weak var tfNightStartHour: UITextField!
    @IBOutlet weak var tfNightStartMinutes: UITextField!
    @IBOutlet weak var tfNightEndMinutes: UITextField!
    @IBOutlet weak var tfNightEndHour: UITextField!
    @IBOutlet weak var tfLimitDay: UITextField!
    @IBOutlet weak var tfLimitMonth: UITextField!
    @IBOutlet weak var tfCountStop: UITextField!
    @IBOutlet weak var tfCountStopNight: UITextField!
    
    @IBOutlet weak var stepperPrice: UIStepper!
    @IBOutlet weak var stepperNightStart: UIStepper!
    @IBOutlet weak var stepperNightStartMinutes: UIStepper!
    @IBOutlet weak var stepperNightEnd: UIStepper!
    @IBOutlet weak var stepperNightEndMinutes: UIStepper!
    @IBOutlet weak var stepperLimitDay: UIStepper!
    @IBOutlet weak var stepperLimitMonth: UIStepper!
    @IBOutlet weak var stepperCountStop: UIStepper!
    @IBOutlet weak var stepperCountStopNight: UIStepper!
    
    @IBOutlet weak var btnSave: UIButton!
    
    var datePickerView = UIDatePicker()
    
    // MARK: - Funkce
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
        
        view.showBlurLoader()
        SenzorAPI.shared.requestSenzor(senzor.idHw!) { (senzor, error) in
            if error == nil {
                self.senzor = senzor
                self.insertValues()
                
                self.view.removeBlurLoader()
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: true), animated: true, completion: nil)
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tfSenzorName.delegate = self
        tfPricePerM3.delegate = self
        tfNightStartHour.delegate = self
        tfNightStartMinutes.delegate = self
        tfNightEndHour.delegate = self
        tfNightEndMinutes.delegate = self
        tfLimitDay.delegate = self
        tfLimitMonth.delegate = self
        tfCountStop.delegate = self
        tfCountStopNight.delegate = self

        localizedStrings()
    }
    
    // MARK: -  Delegovaná funkce komponenty UITextField, pro zachycení události kliknutí na tlačítko "Enter" na klávesnici
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true;
    }
    
    // MARK: - delegované funkce komponenty UITableView
    
    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        switch section {
        case 0:
            return "EDIT_NAME_TITLE".localized
        case 1:
            return "EDIT_PARAMETERS_TITLE".localized
        case 2:
            return "EDIT_LIMITS_TITLE".localized
        default:
            return nil
        }
    }
    
    // MARK: - Privátní funkce
    
    /// Funkce pro získání hodnoty z komponenty DatePickerView
    @objc private func handleDatePicker() {
        let formatter = DateFormatter()
        tfNightStartHour.text = formatter.string(from: datePickerView.date)
    }
    
    /// Funkce pro zachycení změny obsahu komponenty UITextField
    ///
    /// - Parameter sender: Objekt, který událost vyvolal
    @IBAction private func changeValueInTextField(_ sender: UIStepper) {
        let stepperValue = Double(sender.value)
        
        switch sender {
        case stepperPrice:
            tfPricePerM3.text = stepperValue.description
        case stepperNightStart:
            tfNightStartHour.text = stepperValue.description
        case stepperNightStartMinutes:
            tfNightStartMinutes.text = stepperValue.description
        case stepperNightEnd:
            tfNightEndHour.text = stepperValue.description
        case stepperNightEndMinutes:
            tfNightEndMinutes.text = stepperValue.description
        case stepperLimitDay:
            tfLimitDay.text = stepperValue.description
        case stepperLimitMonth:
            tfLimitMonth.text = stepperValue.description
        case stepperCountStop:
            tfCountStop.text = stepperValue.description
        case stepperCountStopNight:
            tfCountStopNight.text = stepperValue.description
        default:
            return
        }
    }
    
    /// Funkce pro zachycení události kliknutí na tlačítko "Odeslat"
    ///
    /// - Parameter sender: Objekt, který událost zaslal
    @IBAction private func saveChanges(_ sender: UIButton) {
        if validateFields() {
            let alertSuccess = UIAlertController(title: "EDIT_ALERT_CONFIRM_EDIT_TITLE".localized, message: "EDIT_ALERT_CONFIRM_EDIT_MESSAGE".localized, preferredStyle: .alert)
            alertSuccess.addAction(UIAlertAction(title: "EDIT_ALERT_CONFIRM_EDIT_BUTTON_CANCEL".localized, style: .destructive, handler: { (action) in
                return
            }))
            alertSuccess.addAction(UIAlertAction(title: "EDIT_ALERT_CONFIRM_EDIT_BUTTON_CONFIRM".localized, style: .default, handler: { (action) in
                
                // Request zadané hodnoty
                let parameters: Parameters = [
                    "nameHw": self.tfSenzorName.text ?? "",
                    "pricePerM3": self.tfPricePerM3.text ?? 0,
                    "implPerLit": self.senzor.implPerLit ?? 0,
                    "countStop": self.tfCountStop.text ?? 0,
                    "countStopNight": self.tfCountStopNight.text ?? 0,
                    "nightStartHour": self.tfNightStartHour.text ?? 0,
                    "nightStartMinute": self.tfNightStartMinutes.text ?? 0,
                    "nightEndHour": self.tfNightEndHour.text ?? 0,
                    "nightEndMinute": self.tfNightEndMinutes.text ?? 0,
                    "limitDay": self.tfLimitDay.text ?? 0,
                    "limitMonth": self.tfLimitMonth.text ?? 0
                ]
                
                SenzorAPI.shared.updateSenzor(id: self.senzor.idHw!, parameters: parameters, completionHandler: { (senzor, error) in
                    if error == nil {
                        self.senzor = senzor
                        
                        let storyboard: UIStoryboard = .detail
                        let viewController = storyboard.instantiateViewController(withIdentifier: "SenzorViewController") as! SenzorViewController
                        viewController.senzor = self.senzor
                        
                        self.show(viewController, sender: nil)
                    } else {
                        self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                    }
                })
            }))
            
            present(alertSuccess, animated: true, completion: nil)
        } else {
            let alertFailure = UIAlertController(title: "EDIT_ALERT_VALIDATION_FAILED_TITLE".localized, message: "EDIT_ALERT_VALIDATION_FAILED_MESSAGE".localized, preferredStyle: .alert)
            alertFailure.addAction(UIAlertAction(title: "EDIT_ALERT_VALIDATION_FAILED_BUTTON".localized, style: .cancel, handler: nil))
            
            present(alertFailure, animated: true, completion: nil)
        }
    }
    

    /// Funkce pro vložení aktuálních hodnot vodoměru do TextFieldů
    private func insertValues() {
        // TextFields
        tfSenzorName.text = senzor.nameHw
        tfPricePerM3.text = Double(senzor.pricePerM3!).description
        tfNightStartHour.text = senzor.nightStartHour!.description
        tfNightStartMinutes.text = senzor.nightStartMinute!.description
        tfNightEndHour.text = senzor.nightEndHour!.description
        tfNightEndMinutes.text = senzor.nightEndMinute!.description
        tfLimitDay.text = Double(senzor.limitDay!).description
        tfLimitMonth.text = Double(senzor.limitMonth!).description
        tfCountStop.text = senzor.countStop!.description
        tfCountStopNight.text = senzor.countStopNight!.description
        
        // Steppers
        stepperPrice.value = senzor.pricePerM3!
        stepperNightStart.value = Double(senzor.nightStartHour!)
        stepperNightStartMinutes.value = Double(senzor.nightStartMinute!)
        stepperNightEnd.value = Double(senzor.nightEndHour!)
        stepperNightEndMinutes.value = Double(senzor.nightEndMinute!)
        stepperLimitDay.value = senzor.limitDay!
        stepperLimitMonth.value = senzor.limitMonth!
        stepperCountStop.value = Double(senzor.countStop!)
        stepperCountStopNight.value = Double(senzor.countStopNight!)
    }
    
    /// Funkce pro validaci hodnot v komponentách UITextField
    ///
    /// - Returns: Booleavská hodnota validních vstupů
    private func validateFields() -> Bool {
        // Validace Textového pole Senzor name
        if tfSenzorName.text == nil {
            return false
        }
        // Validace Textového pole Price per m3
        let priceVal = Double(tfPricePerM3.text!)
        if tfPricePerM3.text == nil || priceVal! < 0 {
            return false
        }
        // Validace Textového pole Limit day
        let limitDayVal = Double(tfLimitDay.text!)
        if limitDayVal! < 0 {
            return false
        }
        // Validace Textového pole limit Month
        let limitMonthVal = Double(tfLimitMonth.text!)
        if limitMonthVal! < 0 {
            return false
        }
        // Validace Textového pole Night End Hour
        let nightEndHour = Double(tfNightEndHour.text!)
        if nightEndHour! < 0 {
            return false
        }
        // Validace Textového pole Night Start Hour
        let nightStartHour = Double(tfNightStartHour.text!)
        if nightStartHour! < 0 {
            return false
        }
        // Validace Textového pole Night Start Hour
        let nightStartMinutes = Double(tfNightStartMinutes.text!)
        if nightStartMinutes! < 0 {
            return false
        }
        // Validace Textového pole Night Start Hour
        let nightEndMinutes = Double(tfNightEndMinutes.text!)
        if nightEndMinutes! < 0 {
            return false
        }
        
        // Validace Textového pole Count Stop
        let countStop = Double(tfCountStop.text!)
        if countStop! < 0 {
            return false
        }
        // Validace Textového pole Count Stop Night
        let countStopNight = Double(tfCountStopNight.text!)
        if countStopNight! < 0 {
            return false
        }
        
        return true
    }
    
    /// Funkce zajišťující naplnění lokalizovaných textů v GUI
    private func localizedStrings() {
        labelPricePerM3.text = "EDIT_LABEL_PRICE".localized
        labelStartNight.text = "EDIT_LABEL_START_NIGHT".localized
        labelStartNightMinutes.text = "EDIT_LABEL_START_NIGHT_MINUTES".localized
        labelEndNight.text = "EDIT_LABEL_END_NIGHT".localized
        labelEndNightMinutes.text = "EDIT_LABEL_END_NIGHT_MINUTES".localized
        labelCountStop.text = "EDIT_LABEL_COUNT_STOP".localized
        labelCountStopNight.text = "EDIT_LABEL_COUNT_STOP_NIGHT".localized
        labelDayLimit.text = "EDIT_LABEL_DAY_LIMIT".localized
        labelMonthLimit.text = "EDIT_LABEL_MONTH_LIMIT".localized
        btnSave.setTitle("BUTTON_SAVE".localized, for: .normal)
        self.title = "EDIT_TITLE".localized
    }
}
