//
//  ServiceSenzorConfiguration.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 02.02.18.
//  Copyright © 2018 Lukáš Šanda. All rights reserved.
//

import Foundation
import Alamofire


// MARK: - Pomocná třída zajišťující aktualizaci hodnot po odeslání požadavku na změnu hodnot
class ServiceSenzorConfiguration {
    
    // MARK: - Proměnné
    
    var senzor: Senzor
    
    init(senzor: Senzor) {
        self.senzor = senzor
    }
    
    // MARK: - Funkce
    
    
    /// Funkce zajišťující aktualizaci
    ///
    /// - Parameters:
    ///   - parameters: Nové hodnoty pro aktualizaci
    ///   - consumption: Nová celková spotřeba
    ///   - completionHandler: Výstupní parametr pro zachycení asynchronního výstupu
    func update(parameters: Parameters = [:], consumption: Double = Double.nan, completionHandler: @escaping (_ responseObject: Senzor?, _ error: Error?) -> ()) {
        let taskGroup = DispatchGroup()
        
        if !parameters.isEmpty {
            taskGroup.enter()
            SenzorAPI.shared.updateSenzor(id: self.senzor.idHw!, parameters: parameters, completionHandler: { (senzor, error) in
                if error == nil {
                    self.senzor.implPerLit = parameters["implPerLit"] as? Int
                    self.senzor.countStop = parameters["countStop"] as? Int
                    self.senzor.countStopNight = parameters["countStopNight"] as? Int
                }
            })
        }
        
        if consumption != Double.nan {
            taskGroup.enter()
            ConsumptionAPI.shared.adjustTotalConsumption(id: self.senzor.idHw!, consumption: consumption) { (senzor, error) in
                if error == nil {
                    self.senzor.consumptionTotal = senzor?.consumptionTotal
                }
            }
        }
        
        taskGroup.notify(queue: DispatchQueue.main, work: DispatchWorkItem(block: {
            completionHandler(self.senzor, nil)
        }))
    }
}
