//
//  ServiceTableViewController.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 27.01.18.
//  Copyright © 2018 Lukáš Šanda. All rights reserved.
//

import UIKit
import Alamofire


class ServiceTableViewController: BaseTableViewController, UITextFieldDelegate {
    
    // MARK: - Proměnné
    
    var senzor: Senzor! = nil
    
    @IBOutlet weak var labelImpPerLit: tableHeaderTitle!
    @IBOutlet weak var labelAdjustTotalCons: tableHeaderTitle!
    
    @IBOutlet weak var tfImpPerLit: UITextField!
    @IBOutlet weak var tfAdjustTotalConsumption: UITextField!
    
    @IBOutlet weak var stepperImpPerLit: UIStepper!
    @IBOutlet weak var stepperAdjustTotalConsumption: UIStepper!
    
    @IBOutlet weak var btnSave: UIButton!
    
    // MARK: - Funkce
  
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
        
        view.showBlurLoader()
        SenzorAPI.shared.requestSenzor(senzor.idHw!) { (senzor, error) in
            if error == nil {
                self.btnSave.isEnabled = true
                self.senzor = senzor
                self.insertValues()
                
                self.view.removeBlurLoader()
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                self.btnSave.isEnabled = false
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        btnSave.backgroundColor = .primaryColor
        btnSave.tintColor = .white
        
        tfImpPerLit.delegate = self
        tfAdjustTotalConsumption.delegate = self
        
        localizedStrings()
    }
    
    // MARK: -  Delegovaná funkce komponenty UITextField, pro zachycení události kliknutí na tlačítko "Enter" na klávesnici
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true;
    }
    
    // MARK: - Delegované funkce komponenty UITableView
    
    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        switch section {
        case 0:
            return "SERVICE_PARAMETERS".localized
        case 1:
            return "SERVICE_TOTAL_CONS".localized
        default:
            return ""
        }
    }
    
    // MARK: - Privátní funkce
    
    /// Funkce pro zachycení události kliknutí na tlačítko "Odeslat"
    ///
    /// - Parameter sender: Objekt, který událost zaslal
    @IBAction private func saveChanges(_ sender: UIButton) {
        if validateFields() {
            let alertSuccess = UIAlertController(title: "EDIT_ALERT_CONFIRM_EDIT_TITLE".localized, message: "EDIT_ALERT_CONFIRM_EDIT_MESSAGE".localized, preferredStyle: .alert)
            alertSuccess.addAction(UIAlertAction(title: "EDIT_ALERT_CONFIRM_EDIT_BUTTON_CONFIRM".localized, style: .default, handler: { (action) in
                
                let serviceConfiguration = ServiceSenzorConfiguration(senzor: self.senzor)
                
                // Post Request zadané hodnoty
                let parameters: Parameters = [
                    "nameHw": self.senzor.nameHw ?? "",
                    "pricePerM3": self.senzor.pricePerM3 ?? 0,
                    "implPerLit": self.tfImpPerLit.text ?? "",
                    "countStop": self.senzor.countStop ?? 0,
                    "countStopNight": self.senzor.countStopNight ?? 0,
                    "nightStartHour": self.senzor.nightStartHour ?? 0,
                    "nightEndHour": self.senzor.nightEndHour ?? 0,
                    "limitDay": self.senzor.limitDay ?? 0.0,
                    "limitMonth": self.senzor.limitMonth ?? 0.0
                ]
                
                
                if self.senzor.consumptionTotal != Double(self.tfAdjustTotalConsumption.text!) {
                    let consumption = Double(self.tfAdjustTotalConsumption.text!)
                    
                    serviceConfiguration.update(parameters: parameters, consumption: consumption!, completionHandler: { (senzor, error) in
                        if error == nil {
                            self.senzor = senzor
                            
                            let storyboard: UIStoryboard = .detail
                            let viewController = storyboard.instantiateViewController(withIdentifier: "SenzorViewController") as! SenzorViewController
                            viewController.senzor = self.senzor
                            
                            self.show(viewController, sender: nil)
                        } else {
                            self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                            self.btnSave.isEnabled = false
                        }
                    })
                } else {
                    serviceConfiguration.update(parameters: parameters, completionHandler: { (senzor, error) in
                        if error == nil {
                            self.senzor = senzor
                            
                            let storyboard: UIStoryboard = .detail
                            let viewController = storyboard.instantiateViewController(withIdentifier: "SenzorViewController") as! SenzorViewController
                            viewController.senzor = self.senzor
                            
                            self.show(viewController, sender: nil)
                        } else {
                            self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                            self.btnSave.isEnabled = false
                        }
                    })
                }
                
            }))
            alertSuccess.addAction(UIAlertAction(title: "EDIT_ALERT_CONFIRM_EDIT_BUTTON_CANCEL".localized, style: .destructive, handler: nil))
            
            present(alertSuccess, animated: true, completion: nil)
        } else {
            let alertFailure = UIAlertController(title: "EDIT_ALERT_VALIDATION_FAILED_TITLE".localized, message: "EDIT_ALERT_VALIDATION_FAILED_MESSAGE".localized, preferredStyle: .alert)
            alertFailure.addAction(UIAlertAction(title: "EDIT_ALERT_VALIDATION_FAILED_BUTTON".localized, style: .cancel, handler: nil))
            
            present(alertFailure, animated: true, completion: nil)
        }
    }
    
    
    /// Funkce pro zachycení změny hodnot v komponentách UITextField
    ///
    /// - Parameter sender: Objekt, který událost zaslal
    @IBAction private func changeValueInTextField(_ sender: UIStepper) {
        let stepperValue = Double(sender.value)
        
        switch sender {
        case stepperImpPerLit:
            tfImpPerLit.text = stepperValue.description
            
        case stepperAdjustTotalConsumption:
            tfAdjustTotalConsumption.text = stepperValue.description
            
        default:
            return
        }
    }
    
    /// Funkce pro validaci hodnot v komponentách UITextField
    ///
    /// - Returns: Booleavská hodnota validních vstupů
    private func validateFields() -> Bool {
        let impPerLitVal = Double(tfImpPerLit.text!)
        if impPerLitVal! < 0 {
            return false
        }
        
        return true
    }
    
    /// Funkce pro vložení aktuálních hodnot vodoměru do komponenty UITextField
    private func insertValues() {
        // TextFields
        tfImpPerLit.text = senzor.implPerLit!.description
        tfAdjustTotalConsumption.text = senzor.consumptionTotal?.description
        // Steppers
        stepperImpPerLit.value = Double(senzor.implPerLit!)
        stepperAdjustTotalConsumption.value = senzor.consumptionTotal!
    }
    
    /// Funkce zajišťující naplnění lokalizovaných textů v GUI
    private func localizedStrings() {
        labelImpPerLit.text = "SERVICE_IMPL_PER_LIT".localized
        labelAdjustTotalCons.text = "SERVICE_ADJUST_TITLE".localized
        btnSave.setTitle("BUTTON_SAVE".localized, for: .normal)
        self.title = "SERVICE_TITLE".localized
    }
}
