

//
//  ValveStateViewController.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 16.12.17.
//  Copyright © 2017 Lukáš Šanda. All rights reserved.
//

import UIKit


class ValveStateViewController: BaseViewController {
    
    // MARK: Proměnné
    
    var senzor: Senzor! = nil
    
    @IBOutlet weak var labelValveTitle: UILabel!
    @IBOutlet weak var btnState: UIButton!
    @IBOutlet weak var labelStateTitle: UILabel!
    @IBOutlet weak var labelState: UILabel!
    @IBOutlet weak var labelChangerTitle: UILabel!
    @IBOutlet weak var labelChanger: UILabel!
    @IBOutlet weak var labelChangeDateTitle: UILabel!
    @IBOutlet weak var labelChangeDate: UILabel!
    
    // MARK: - Funkce
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
        
        view.showBlurLoader()
        SenzorAPI.shared.requestSenzor(senzor.idHw!) { (senzor, error) in
            if error == nil {
                if let senzor = senzor {
                    self.btnState.isEnabled = true
                    self.senzor = senzor
                    self.insertValues()
                    
                    self.view.removeBlurLoader()
                }
            } else {
                self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                self.btnState.isEnabled = false
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        localizedStrings()
    }
    
    // MARK: - Privátní funkce
    
    /// Funkce pro zachycení události kliknutí na tlačítko "Uložit"
    ///
    /// - Parameter sender: Objekt, který událost zaslal
    @IBAction func btnStateChange(_ sender: UIButton) {
        if !ReachabilityManager.shared.isConnectedToNetwork() {
            self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
            self.btnState.isEnabled = false
        }
        
        switch senzor.state {
        case 1:
            self.btnAppereanceForState(4)
            self.labelState.text = Utils.shared.convertStateToString(4)
            self.labelChanger.text = UserDefaults.standard.userFullName
            self.labelChangeDate.text = Utils.shared.formatDate(Utils.shared.getCurrentDateTime(), format: "dd.\u{00a0}MM.\u{00a0}YYYY HH:mm:ss")
            
            SenzorAPI.shared.putValveState(id: self.senzor.idHw!, state: 4, completionHandler: { (senzor, error) in
                if error == nil {
                    if let senzor = senzor {
                        self.senzor = senzor
                        VodomerData.shared.storeSenzor(senzor)
                        
                        self.checkStateFor(self.senzor.state!)
                    }
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                    self.btnState.isEnabled = false
                }
            })
            
        case 3:
            self.btnAppereanceForState(2)
            self.labelState.text = Utils.shared.convertStateToString(2)
            self.labelChanger.text = UserDefaults.standard.userFullName
            self.labelChangeDate.text = Utils.shared.formatDate(Utils.shared.getCurrentDateTime(), format: "dd.\u{00a0}MM.\u{00a0}YYYY HH:mm:ss")
            
            SenzorAPI.shared.putValveState(id: self.senzor.idHw!, state: 2, completionHandler: { (senzor, error) in
                if error == nil {
                    if let senzor = senzor {
                        VodomerData.shared.storeSenzor(senzor)
                        
                        self.checkStateFor(self.senzor.state!)
                    }
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                    self.btnState.isEnabled = false
                }
            })
            
        default:
            return
        }
    }
    
    /// Funkce kontrolující změnu stavu
    ///
    /// - Parameter sender: Stav, který má být zjišťován
    private func checkStateFor(_ state: Int) {
        let delay = DispatchTime.now() + 3
        
        DispatchQueue.main.asyncAfter(deadline: delay) {
            SenzorAPI.shared.requestSenzor(self.senzor.idHw!, completionHandler: { (senzor, error) in
                if error == nil {
                    if let senzor = senzor {
                        switch self.senzor.state {
                        // Otevřený
                        case 1:
                            self.senzor.state = senzor.state
                            self.btnAppereanceForState(self.senzor.state!)
                            
                            self.labelState.text = Utils.shared.convertStateToString(senzor.state)
                            self.labelChangeDate.text = Utils.shared.formatDate(Utils.shared.getCurrentDateTime(), format: "dd.\u{00a0}MM.\u{00a0}YYYY HH:mm:ss")
                            return
                        // Čeká na oteření
                        case 2:
                            self.checkStateFor(self.senzor.state!)
                        // Uzavřený
                        case 3:
                            self.senzor.state = senzor.state
                            self.btnAppereanceForState(self.senzor.state!)
                            
                            self.labelState.text = Utils.shared.convertStateToString(senzor.state)
                            self.labelChangeDate.text = Utils.shared.formatDate(Utils.shared.getCurrentDateTime(), format: "dd.\u{00a0}MM.\u{00a0}YYYY HH:mm:ss")
                            return
                        // Čeká na uzavření
                        case 4:
                            self.checkStateFor(self.senzor.state!)
                        default:
                            return
                        }
                    }
                } else {
                    self.present(ReachabilityManager.shared.noConnectionAlert(showLogin: false), animated: true, completion: nil)
                    self.btnState.isEnabled = false
                }
            })
        }
    }
    
    /// Pomocná funkce zajišťující naplnění komponent GUI
    private func insertValues() {
        btnAppereanceForState(senzor.state!)
        
        labelState.text = Utils.shared.convertStateToString(senzor.state)
        labelChanger.text = senzor.stateModifierUserName
        labelChangeDate.text = Utils.shared.formatDate(senzor.stateModifiedDate!, format: "dd.\u{00a0}MM.\u{00a0}YYYY HH:mm:ss")
    }
    
    /// Pomocná funkce zajišťující změnu GUI na základě stavu senzoru
    ///
    /// - Parameter sender: Stav, pro který má být okno překresleno
    private func btnAppereanceForState(_ state: Int) {
        switch state {
        case 1:
            btnState.backgroundColor = .primaryColor
            btnState.setTitle("VALVE_CONTROL_BUTTON_TITLE_OPEN".localized, for: .normal)
            btnState.isEnabled = true
        case 2:
            btnState.backgroundColor = .orange
            btnState.setTitle("VALVE_CONTROL_BUTTON_TITLE_OPENING".localized, for: .normal)
            btnState.isEnabled = false
        case 3:
            btnState.backgroundColor = .red
            btnState.setTitle("VALVE_CONTROL_BUTTON_TITLE_CLOSE".localized, for: .normal)
            btnState.isEnabled = true
        case 4:
            btnState.backgroundColor = .orange
            btnState.setTitle("VALVE_CONTROL_BUTTON_TITLE_CLOSING".localized, for: .normal)
            btnState.isEnabled = false
        default:
            return
        }
    }
    
    /// Funkce zajišťující naplnění lokalizovaných textů v GUI
    private func localizedStrings() {
        labelValveTitle.text = "VALVE_LABEL_TITLE".localized
        labelStateTitle.text = "VALVE_STATE_TITLE".localized
        labelChangerTitle.text = "VALVE_CHANGER_TITLE".localized
        labelChangeDateTitle.text = "VALVE_CHANGE_DATE_TITLE".localized
        self.title = "VALVE_TITLE".localized
    }
}
