//
//  Formatter.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 09.02.18.
//  Copyright © 2018 Lukáš Šanda. All rights reserved.
//

import Foundation

class Formatter {
    
    fileprivate struct DefaultFormatters {
        
        static fileprivate var dayDateFormatter: DateFormatter = {
            let formatter = DateFormatter()
            formatter.setLocalizedDateFormatFromTemplate("EEEE")
            formatter.dateFormat = "EEEE"
            return formatter
        }()
        
        static fileprivate var dateFormatter: DateFormatter = {
            let formatter = DateFormatter()
            formatter.dateStyle = .medium
            formatter.timeStyle = .none
            
            return formatter
        }()
        
        static fileprivate var fullDateFormatter: DateFormatter = {
            let dateFormatter = DateFormatter()
            dateFormatter.dateStyle = .full
            dateFormatter.doesRelativeDateFormatting = false
            
            return dateFormatter
        }()
        
        static fileprivate var fullDateRelativeFormatter: DateFormatter = {
            let dateFormatter = DateFormatter()
            dateFormatter.dateStyle = .full
            dateFormatter.doesRelativeDateFormatting = true
            
            return dateFormatter
        }()
        
        static fileprivate var shortDigitDurationFormatter: DateComponentsFormatter = {
            let formatter = DateComponentsFormatter()
            formatter.zeroFormattingBehavior = .pad
            formatter.allowedUnits = [.minute, .second]
            
            return formatter
        }()
        
        static fileprivate var timeFormatter: DateFormatter = {
            let formatter = DateFormatter()
            formatter.timeStyle = .full
            
            return formatter
        }()
        
        static fileprivate var durationFormatter: DateComponentsFormatter = {
            let formatter = DateComponentsFormatter()
            formatter.unitsStyle = .full
            formatter.allowedUnits = [.minute, .second]
            
            return formatter
        }()
    }
    
    // 7:45 (AM)
    static func formatTimeFromDate(_ date: Date) -> String {
        return DefaultFormatters.timeFormatter.string(from: date)
    }
    
    // "Today" / Monday 5. 2. 2018
    static func formatFullDate(_ date: Date, relative: Bool) -> String {
        if relative {
            return DefaultFormatters.fullDateRelativeFormatter.string(from: date)
        } else {
            return DefaultFormatters.fullDateFormatter.string(from: date)
        }
    }
}
