//
//  ReachabilityManager.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 24.03.18.
//  Copyright (c) 2014, Ashley Mills. All rights reserved.
//
// https://github.com/ashleymills/Reachability.swift/blob/master/Reachability/Reachability.swift

import SystemConfiguration
import UIKit


class ReachabilityManager {
    
    // MARK: - Proměnné
    
    let timer = Timer()
    
    /// Sdílená proměnná
    static let shared: ReachabilityManager = { return ReachabilityManager() }()
    
    // MARK: - Funkce
    
    
    /// Funkce zjišťující stav internetu
    ///
    /// - Returns: Booleovská hodnota, zda má App přístup k internetu
    func isConnectedToNetwork() -> Bool {
        var zeroAddress = sockaddr_in(sin_len: 0, sin_family: 0, sin_port: 0, sin_addr: in_addr(s_addr: 0), sin_zero: (0, 0, 0, 0, 0, 0, 0, 0))
        zeroAddress.sin_len = UInt8(MemoryLayout.size(ofValue: zeroAddress))
        zeroAddress.sin_family = sa_family_t(AF_INET)
        
        let defaultRouteReachability = withUnsafePointer(to: &zeroAddress) {
            $0.withMemoryRebound(to: sockaddr.self, capacity: 1) {zeroSockAddress in
                SCNetworkReachabilityCreateWithAddress(nil, zeroSockAddress)
            }
        }
        
        var flags: SCNetworkReachabilityFlags = SCNetworkReachabilityFlags(rawValue: 0)
        if SCNetworkReachabilityGetFlags(defaultRouteReachability!, &flags) == false {
            return false
        }
        
        let isReachable = (flags.rawValue & UInt32(kSCNetworkFlagsReachable)) != 0
        let needsConnection = (flags.rawValue & UInt32(kSCNetworkFlagsConnectionRequired)) != 0
        let ret = (isReachable && !needsConnection)
        
        return ret
    }
    
    
    /// Pomocná funkce, která vytváří komponnetu uIAlertController, pro informování o absenci internetového připojení
    ///
    /// - Parameter showLogin: rozhodnutí, zda má app přejít na přihlašovací okno
    /// - Returns: Vytvořená komponenta UIAlertController
    func noConnectionAlert(showLogin: Bool) -> UIAlertController {
        let alert = UIAlertController(title: "LOGIN_ALERT_NO_CONNECTION_TITLE".localized, message: "LOGIN_ALERT_NO_CONNECTION_MESSAGE".localized, preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "LOGIN_ALERT_NO_CONNECTION_BUTTON_TITLE".localized, style: .cancel, handler: { _ in
            if showLogin {
                let storyboard: UIStoryboard = .main
                let viewController = storyboard.instantiateViewController(withIdentifier: "LoginViewController") as! LoginViewController
                
                if let view = UIApplication.topViewController() {
                    view.view.removeBlurLoader()
                    view.show(viewController, sender: self)
                }
            }
        }))

        return alert
    }
    
}

// MARK: - Rozšíření pro Pod Reachability
extension UIApplication {
    
    static func topViewController(base: UIViewController? = UIApplication.shared.delegate?.window??.rootViewController) -> UIViewController? {
        
        if let nav = base as? UINavigationController {
            return topViewController(base: nav.visibleViewController)
        }
        
        if let tab = base as? UITabBarController, let selected = tab.selectedViewController {
            return topViewController(base: selected)
        }
        
        if let presented = base?.presentedViewController {
            return topViewController(base: presented)
        }
        
        return base
    }
}
