//
//  HydrometraData.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 01.09.17.
//  Copyright © 2017 Univerzita Pardubice. All rights reserved.
//

import Foundation
import Alamofire


// MARK: - Pomocná třída obsahující funkce zajišťující práci s daty
/// Třída obalující práci s daty senzoru
class VodomerData: NSObject {
    
    // MARK: - Proměnné
    var senzors = [Senzor]()
    
    /// Poslední prohlížený senzor
    var tempSenzor: Senzor! = nil
    
    /// Sdílená proměnná
    static let shared: VodomerData = { return VodomerData() }()
    
    // MARK: - Funkce, která v hlavním vlákně provede requesty na spotřeby, které jsou nutné pro první načtení view
    /**
     Funkce, která v hlavním vlákně provede requesty na spotřeby, které jsou nutné pro první načtení view
     
     - Parameter id: ID požadovaného senzoru
     - Parameter completionHandler: zajištění dokončení v hlavním vlákně
     
     - Returns: Úspěch – vodoměr, Neúspěch – Error
     */
    func defaultInitConsumption(id: Int, completionHandler: @escaping (_ responseObject: Senzor?, _ error: Error?) -> ()) {
        let taskGroup = DispatchGroup()
        var selectedSenzor: Senzor!
        
        for (index, senzor) in self.senzors.enumerated() {
            if self.senzors[index].idHw == id {
                selectedSenzor = senzor
            }
        }
        taskGroup.enter()
        ConsumptionAPI.shared.requestConsumption(id: id, offset: 0, timeUnit: .day) { (senzor, error) in
            if error == nil {
                selectedSenzor.consumptionDay = senzor?.consumptionDay
                taskGroup.leave()
            } else {
                completionHandler(nil, error)
            }
        }
        
        taskGroup.enter()
        ConsumptionAPI.shared.requestConsumption(id: id, offset: 0, timeUnit: .month) { (senzor, error) in
            if error == nil {
                selectedSenzor.consumptionMonth = senzor?.consumptionMonth
                taskGroup.leave()
            } else {
                completionHandler(nil, error)
            }
        }
        
        taskGroup.notify(queue: DispatchQueue.main, work: DispatchWorkItem(block: {
            completionHandler(selectedSenzor, nil)
        }))
    }
    
    // MARK: - Pomocná funkce pro uložení vodoměru do paměti telefonu
    /**
     Pomocná funkce pro uložení vodoměru do paměti telefonu
     
     - Parameter newSenzor: Senzor příslušný pro uložení do paměti
     */
    func storeSenzor(_ newSenzor: Senzor) {
        for (index, senzor) in VodomerData.shared.senzors.enumerated() {
            if senzor.idHw == newSenzor.idHw {
                VodomerData.shared.senzors[index] = newSenzor
            }
        }
    }
    
    func totalConsumption(completionHandler: @escaping (_ error: Error?) -> ()) {
        for senzor in senzors {
            ConsumptionAPI.shared.requestTotalConsumption(id: senzor.idHw!) { (consumption, error) in
                if error == nil {
                    senzor.consumptionTotal = consumption
                    
                    completionHandler(nil)
                } else {
                    completionHandler(error)
                }
            }
        }
    }
}

// MARK: - Vlastní Error, který lze inicializovat vlastní zprávou
/// Vlastní Error, který lze inicializovat vlastní zprávou
struct RuntimeError: Error {
    let message: String
    
    init(_ message: String) {
        self.message = message
    }
    
    // MARK: Metoda pro možnost implementace vícejazyčné aplikace
    /// Metoda pro možnost implementace vícejazyčné aplikace
    public var localizedDescription: String {
        return message
    }
}

// MARK: - Období, jímž disponují data vztahující se k vodoměru
/// Období, jímž disponují data vztahující se k vodoměru
public enum TimeUnit: String {
    case day = "Day"
    case month = "Month"
    case year = "Year"
}
