//
//  AuthenticationAPI.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 09.02.18.
//  Copyright © 2018 Lukáš Šanda. All rights reserved.
//

import Foundation
import Alamofire


class AuthenticationAPI: AlamofireManager {
    
    // MARK: Sdílená proměnná
    static let shared: AuthenticationAPI = { return AuthenticationAPI() }()
    
    // MARK: - Funkce
    
    /// Funkce pro ověření uživatele vůči aplikaci
    ///
    /// - Parameters:
    ///   - username: Uživatelské jméno
    ///   - password: Uživatelské heslo
    ///   - completionHandler: Výstupní parametr pro zachycení asynchronního výstupu
    func login(username: String, password: String, completionHandler: @escaping (_ responseObject: String?, _ error: Error?) -> ()) {
        let url = "\(baseURL)/api-m/auth/token"
        
        let parameters: Parameters = [
            "username" : username,
            "password" : password
        ]
        
        headers = ["Accept": "application/json"]
        
        Manager.request(url, method: .post, parameters: parameters, encoding: URLEncoding.default, headers: headers).responseJSON { (response) in
            switch response.result {
            case .success:
                if response.response?.statusCode == 200, let json = response.result.value as? NSDictionary {
                    completionHandler(json["token"] as? String, nil)
                } else {
                    completionHandler(nil, RuntimeError("Chyba při dynamickém přetypování."))
                }
            case .failure(let error):
                completionHandler(nil, error)
            }
        }
    }
    
}


