//
//  CollectionCell.swift
//  Hydrometra
//
//  Created by Lukáš Šanda on 04.04.18.
//  Copyright © 2018 Lukáš Šanda. All rights reserved.
//

import UIKit


/// Třída, která obsluhuje buňku kolekce senzorů
class SenzorCell: UITableViewCell {
    
    // MARK: - Proměnné
    
    @IBOutlet weak var labelTitle: UILabel!
    
    @IBOutlet weak var labelValveStateTitle: DescriptionContent!
    @IBOutlet weak var labelConsumptionTitle: DescriptionContent!
    
    @IBOutlet weak var labelSenzorState: UILabel!
    @IBOutlet weak var labelValveState: UILabel!
    @IBOutlet weak var labelConsumption: UILabel!
    @IBOutlet weak var consumptionStack: UIStackView!
    
    // MARK: - Funkce
    
    override func awakeFromNib() {
        super.awakeFromNib()
        
        self.backgroundColor = .primaryColor
        self.selectionStyle = .none
    }
    
    
    /// Funkce, která naplní hodnoty v buňce
    ///
    /// - Parametr senzor: Zdroj pro hodnoty
    func configure(for senzor: Senzor) {
        // Nastavení hodnot do příslušných labelů
        
        // Název
        if let title = senzor.nameHw, let isOnline = senzor.isOnline, let lastTimeOnline = senzor.lastTimeOnline {
            let baseString = "◉ \(title)"
            let attributedString = NSMutableAttributedString(string: baseString, attributes: nil)
            let dotRange = (attributedString.string as NSString).range(of: "◉")
            
            labelValveStateTitle.text = "SENZOR_CELL_VALVE_TITLE".localized
            labelConsumptionTitle.text = "SENZOR_CELL_CONSUMPTION_TITLE".localized
            
            labelSenzorState.text = { () -> String in
                if isOnline {
                    attributedString.addAttribute(.foregroundColor, value: UIColor.green, range: dotRange)
                    labelTitle.attributedText = attributedString
                    
                    return "COLLECTION_SENZOR_ACTIVE".localized
                } else {
                    attributedString.addAttribute(.foregroundColor, value: UIColor.red, range: dotRange)
                    labelTitle.attributedText = attributedString
                    
                    let formatedDate = Utils.shared.formatDate(lastTimeOnline, format: "dd.\u{00a0}MM.\u{00a0}YYYY HH:mm:ss") ?? ""
                    return ("COLLECTION_SENZOR_INACTIVE".localized + "\(formatedDate)")
                }
            }()
            labelValveState.text = (senzor.state == 1) ? "COLLECTION_SENZOR_OPEN".localized : "COLLECTION_SENZOR_CLOSED".localized
            if UserDefaults.standard.userRole == "admin", let totalConsumption = senzor.consumptionTotal {
                labelConsumption.text = "\(String(format: "%.3f", totalConsumption)) m³"
                consumptionStack.isHidden = false
            } else {
                consumptionStack.isHidden = true
            }
        }
        
        
    }
    
}
